/*--------------------------------------------------------------------------+
$Id: CLASS.java 11213 2016-03-04 16:27:08Z MONDAL $
|                                                                          |
| Copyright 2016 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.spiderchart.example;

import java.util.function.Supplier;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.fortiss.tooling.spiderchart.builder.AxesConfigurer;
import org.fortiss.tooling.spiderchart.builder.SpiderChartBuilder;
import org.fortiss.tooling.spiderchart.sequence.LineDataSeq;
import org.fortiss.tooling.spiderchart.swt.SpiderChartViewer;

@SuppressWarnings("javadoc")
public final class Sample {

	private enum Brand {
		COMMUNAL, INTERNATIONAL, LOCAL, OUT_OF_MARKET, STANDARD
	}

	private static SpiderChartViewer viewer;

	private static void buildSpiderChart(final Shell shell) {
		final Supplier<Object> iPhoneData = IPhone::new;
		final Supplier<Object> nexusData = Nexus::new;

		viewer =
				SpiderChartBuilder.config(
						shell,
						settings -> {
							settings.title(title -> title.setText("Smartphone Comparison Scale"))
									.legend(legend -> {
										legend.addItem(iPhoneData);
										legend.addItem(nexusData);
									})
									.plotter(
											plotter -> {
												final AxesConfigurer configuration =
														new AxesConfigurer.Builder()
																.addAxis("Battery", 5, 0)
																.addAxis("Camera", 5, 0)
																.addAxis("Display", 5, 0)
																.addAxis("Memory", 5, 0)
																.addAxis("Brand", 5, 0).build();
												plotter.use(configuration);
											});
						}).viewer(
						chart -> {
							chart.data(firstData -> firstData.inject(iPhoneData)).data(
									secondData -> secondData.inject(nexusData));
						});

		// Updating the chart with new parameters
		Display.getDefault().asyncExec(() -> {
			// changing values in runtime
				final LineDataSeq iPhoneDataSequence =
						LineDataSeq.of(iPhoneData.get(), 2.0, 4.2, 4.1, 42.8, 3.7,
								Brand.INTERNATIONAL);
				// Set the first sequence
				viewer.getChart().getSpiderPlotter().setSeq(0, iPhoneDataSequence);

				// changing axes in runtime
				final AxesConfigurer configuration =
						new AxesConfigurer.Builder().addAxis("Battery", 5, 0)
								.addAxis("Screen", 5, 0).addAxis("Display", 5, 0)
								.addAxis("Memory", 50, 0).addAxis("Sound", 5, 0)
								.addAxis("Brand", Brand.class).build();

				final LineDataSeq nexusDataSequence =
						LineDataSeq.of(nexusData.get(), 2.4, 3.2, 2.1, 23.8, 1.7, Brand.LOCAL);

				// Set the second sequence
				viewer.getChart().getSpiderPlotter().setSeq(1, nexusDataSequence);
				viewer.getChart().getSpiderPlotter().use(configuration);
				viewer.getChart().getSpiderPlotter().setMarkScalesOnEveryAxis(true);
			});
	}

	public static void main(final String[] args) {
		final Display display = new Display();
		final Shell shell = new Shell(display, SWT.DIALOG_TRIM);
		shell.setSize(800, 750);

		buildSpiderChart(shell);
		shell.open();

		while(!shell.isDisposed()) {
			if(!display.readAndDispatch()) {
				display.sleep();
			}
		}

		viewer.getChart().stopWorker();
		viewer.getChart().dispose();

		display.dispose();
	}

}
