/*--------------------------------------------------------------------------+
$Id: CLASS.java 11213 2016-03-04 16:27:08Z MONDAL $
|                                                                          |
| Copyright 2016 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.spiderchart.util;

import java.util.Arrays;
import java.util.List;

/**
 * Utility methods for Spider Charts
 *
 * @author AMIT KUMAR MONDAL
 *
 */
public final class ChartUtil {

	/**
	 * Returns the array of string represented constants of any Enum
	 */
	public static <E extends Enum<E>> List<String> enumConstants(final Class<E> value) {
		final String[] constants = new String[value.getEnumConstants().length];
		int i = 0;
		for(final Enum<E> enumVal : value.getEnumConstants()) {
			constants[i++] = enumVal.name();
		}
		return Arrays.asList(constants);
	}

	/** Returns the primitive double array from wrapper double array */
	public static double[] toDoublePrimitiveArray(final Double[] wrappedArray) {
		final double[] array = new double[wrappedArray.length];
		for(int i = 0; i < wrappedArray.length; i++) {
			array[i] = wrappedArray[i].intValue();
		}
		return array;
	}

	/**
	 * Wraps a list of values and converts it to its corresponding double values
	 */
	public static double[] wrapValues(final Object[] values) {
		final double[] wrappedValues = new double[values.length];

		int i = 0;
		for(final Object value : values) {
			if(value instanceof Double || value instanceof Integer) {
				wrappedValues[i++] = (double)value;
			}
			if(value instanceof Long) {
				Long l = new Long((long)value);
				wrappedValues[i++] = l.doubleValue();
			}
			if(value instanceof Enum<?>) {
				final Enum<?> enumConst = (Enum<?>)value;
				wrappedValues[i++] = enumConst.ordinal() + 1;
			}
		}
		return wrappedValues;
	}

	/**
	 * Constructor
	 */
	private ChartUtil() {
	}

}
